@@ -16,23 +16,29 @@
 # limitations under the License.
 
 """
-Methods that convert incoming data to dataflows.
+Classes to load data and produce dataflows
 """
 
+from __future__ import annotations
+
 import itertools
 import json
 import os
 from collections import defaultdict
-from typing import DefaultDict, Dict, List, Optional, Sequence, Union
+from pathlib import Path
+from typing import Any, DefaultDict, Dict, Iterator, List, Optional, Sequence, TextIO, Union
 
 from jsonlines import Reader, Writer
+from tabulate import tabulate
+from termcolor import colored
 
 from ..utils.context import timed_operation
-from ..utils.detection_types import JsonDict, Pathlike
+from ..utils.error import FileExtensionError
 from ..utils.identifier import get_uuid_from_str
 from ..utils.pdf_utils import PDFStreamer
 from ..utils.tqdm import get_tqdm
-from ..utils.utils import FileExtensionError, is_file_extension
+from ..utils.types import JsonDict, PathLikeOrStr
+from ..utils.utils import is_file_extension
 from .base import DataFlow
 from .common import FlattenData, JoinData, MapData
 from .custom import CacheData, CustomDataFromIterable, CustomDataFromList
@@ -49,41 +55,104 @@ def _reset_df_and_get_length(df: DataFlow) -> int:
     return length
 
 
+class FileClosingIterator:
+    """
+    A custom iterator that closes the file object once the iteration is complete.
+
+    This iterator is used to ensure that the file object is properly closed after
+    reading the data from it. It is used in the context of reading data from a file
+    in a streaming manner, where the data is not loaded into memory all at once.
+
+    Example:
+        ```python
+        file = open(path, "r")
+        iterator = Reader(file)
+        closing_iterator = FileClosingIterator(file, iter(iterator))
+
+        df = CustomDataFromIterable(closing_iterator, max_datapoints=max_datapoints)
+        ```
+
+    """
+
+    def __init__(self, file_obj: TextIO, iterator: Iterator[Any]):
+        """
+        Initializes the FileClosingIterator with a file object and its iterator.
+
+        Args:
+            file_obj: The file object to read data from.
+            iterator: The actual iterator of the file object.
+        """
+        self.file_obj = file_obj
+        self.iterator = iterator
+
+    def __iter__(self) -> FileClosingIterator:
+        """
+        Returns the iterator object itself.
+
+        Returns:
+            FileClosingIterator: The instance of the class itself.
+        """
+        return self
+
+    def __next__(self) -> Any:
+        """
+        Returns the next item from the file object's iterator.
+        Closes the file object if the iteration is finished.
+
+        Returns:
+            The next item from the file object's iterator.
+
+        Raises:
+            StopIteration: If there are no more items to return.
+        """
+        try:
+            return next(self.iterator)
+        except StopIteration as exc:
+            self.file_obj.close()
+            raise StopIteration from exc
+
+
 class SerializerJsonlines:
     """
-    Serialize a dataflow from a jsonlines file. Alternatively, save a dataflow of JSON objects to a .jsonl file.
+    Serialize a dataflow from a jsonlines file. Alternatively, save a dataflow of `JSON` objects to a `.jsonl` file.
 
-    **Example:**
+    Example:
+        ```python
+          df = SerializerJsonlines.load("path/to/file.jsonl")
+          df.reset_state()
 
-            df = SerializerJsonlines.load("path/to/file.jsonl")
-            df.reset_state()
-            for dp in df:
-               ... # is a dict
+          for dp in df:
+              ... # is a dict
+        ```
     """
 
     @staticmethod
-    def load(path: Pathlike, max_datapoints: Optional[int] = None) -> CustomDataFromIterable:
+    def load(path: PathLikeOrStr, max_datapoints: Optional[int] = None) -> CustomDataFromIterable:
         """
-        :param path: a path to a .jsonl file.
-        :param max_datapoints: Will stop the iteration once max_datapoints have been streamed
+        Args:
+            path: a path to a .jsonl file.
+            max_datapoints: Will stop the iteration once max_datapoints have been streamed
 
-        :return: dataflow to iterate from
+        Returns:
+            Dataflow to iterate from
         """
         file = open(path, "r")  # pylint: disable=W1514,R1732
         iterator = Reader(file)
-        return CustomDataFromIterable(iterator, max_datapoints=max_datapoints)
+        closing_iterator = FileClosingIterator(file, iter(iterator))
+        return CustomDataFromIterable(closing_iterator, max_datapoints=max_datapoints)
 
     @staticmethod
-    def save(df: DataFlow, path: Pathlike, file_name: str, max_datapoints: Optional[int] = None) -> None:
+    def save(df: DataFlow, path: PathLikeOrStr, file_name: str, max_datapoints: Optional[int] = None) -> None:
         """
-        Writes a dataflow iteratively to a .jsonl file. Every datapoint must be a dict where all items are serializable.
-        As the length of the dataflow cannot be determined in every case max_datapoint prevents generating an
-        unexpectedly large file
+        Writes a dataflow iteratively to a `.jsonl` file. Every datapoint must be a dict where all items are
+        serializable. As the length of the dataflow cannot be determined in every case max_datapoint prevents
+        generating an unexpectedly large file
 
-        :param df: The dataflow to write from.
-        :param path: The path, the .jsonl file to write to.
-        :param file_name: name of the target file.
-        :param max_datapoints: maximum number of datapoint to consider writing to a file.
+        Args:
+            df: The dataflow to write from.
+            path: The path, the .jsonl file to write to.
+            file_name: name of the target file.
+            max_datapoints: maximum number of datapoint to consider writing to a file.
         """
 
         if not os.path.isdir(path):
@@ -106,39 +175,43 @@ class SerializerJsonlines:
 class SerializerTabsepFiles:
     """
     Serialize a dataflow from a tab separated text file. Alternatively, save a dataflow of plain text
-    to a .txt file.
+    to a `.txt` file.
 
-    **Example**:
-
-            df = SerializerTabsepFiles.load("path/to/file.txt")
+    Example:
+        ```python
+        df = SerializerTabsepFiles.load("path/to/file.txt")
 
         will yield each text line of the file.
+        ```
     """
 
     @staticmethod
-    def load(path: Pathlike, max_datapoins: Optional[int] = None) -> CustomDataFromList:
+    def load(path: PathLikeOrStr, max_datapoints: Optional[int] = None) -> CustomDataFromList:
         """
-        :param path: a path to a .txt file.
-        :param max_datapoins: Will stop the iteration once max_datapoints have been streamed
+        Args:
+            path: a path to a .txt file.
+            max_datapoints: Will stop the iteration once max_datapoints have been streamed
 
-        :return: dataflow to iterate from
+        Returns:
+            Dataflow to iterate from
         """
 
         with open(path, "r", encoding="UTF-8") as file:
             file_list = file.readlines()
-        return CustomDataFromList(file_list, max_datapoints=max_datapoins)
+        return CustomDataFromList(file_list, max_datapoints=max_datapoints)
 
     @staticmethod
-    def save(df: DataFlow, path: Pathlike, file_name: str, max_datapoints: Optional[int] = None) -> None:
+    def save(df: DataFlow, path: PathLikeOrStr, file_name: str, max_datapoints: Optional[int] = None) -> None:
         """
         Writes a dataflow iteratively to a .txt file. Every datapoint must be a string.
         As the length of the dataflow cannot be determined in every case max_datapoint prevents generating an
         unexpectedly large file
 
-        :param df: The dataflow to write from.
-        :param path: The path, the .txt file to write to.
-        :param file_name: name of the target file.
-        :param max_datapoints: maximum number of datapoint to consider writing to a file.
+        Args:
+            df: The dataflow to write from.
+            path: The path, the .txt file to write to.
+            file_name: Name of the target file.
+            max_datapoints: Maximum number of datapoint to consider writing to a file.
         """
 
         if not os.path.isdir(path):
@@ -160,11 +233,18 @@ class SerializerFiles:
     """
     Serialize files from a directory and all subdirectories. Only one file type can be serialized. Once specified, all
     other types will be filtered out.
+
+    Example:
+        ```python
+        df = SerializerFiles.load("path/to/dir",file_type=".pdf")
+
+        will yield absolute paths to all `.pdf` files in the directory and all subdirectories.
+        ```
     """
 
     @staticmethod
     def load(
-        path: Pathlike,
+        path: PathLikeOrStr,
         file_type: Union[str, Sequence[str]],
         max_datapoints: Optional[int] = None,
         shuffle: Optional[bool] = False,
@@ -174,22 +254,29 @@ class SerializerFiles:
         Generates a dataflow where a datapoint consists of a string of names of files with respect to some file type.
         If you want to load the files you need to do this in a following step by yourself.
 
-        :param path: A path to some base directory. Will inspect all subdirectories, as well
-        :param file_type: A file type (suffix) to look out for (single str or list of stings)
-        :param max_datapoints: Stop iteration after passing max_datapoints
-        :param shuffle: Shuffle the files, so that the order of appearance in dataflow is random.
-        :param sort: If set to "True" it will sort all selected files by its string
-        :return: dataflow to iterate from
+        Args:
+            path: A path to some base directory. Will inspect all subdirectories, as well
+            file_type: A file type (suffix) to look out for (single str or list of stings)
+            max_datapoints: Stop iteration after passing max_datapoints
+            shuffle: Shuffle the files, so that the order of appearance in dataflow is random.
+            sort: If set to `True` it will sort all selected files by its string
+
+        Returns:
+            Dataflow to iterate from
         """
         df: DataFlow
         df1: DataFlow
         df2: DataFlow
         df3: DataFlow
 
+        path = Path(path)
+        if not path.exists():
+            raise NotADirectoryError(f"The path {path} to the directory or file does not exist")
+
         if shuffle:
             sort = False
-        it1 = os.walk(path, topdown=False)
-        it2 = os.walk(path, topdown=False)
+        it1 = os.walk(os.fspath(path), topdown=False)
+        it2 = os.walk(os.fspath(path), topdown=False)
         df1 = CustomDataFromIterable(it1)
         df2 = CustomDataFromIterable(it2)
         df1 = MapData(df1, lambda dp: None if len(dp[2]) == 0 else dp)
@@ -217,19 +304,19 @@ class SerializerFiles:
         """
         Not implemented
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class CocoParser:
     """
-    A simplified version of the Microsoft COCO helper class for reading  annotations. It currently supports only
+    A simplified version of the COCO helper class for reading  annotations. It currently supports only
     bounding box annotations
 
-    :param annotation_file: location of annotation file
+    Args:
+        annotation_file: Location of annotation file
     """
 
-    def __init__(self, annotation_file: Optional[Pathlike] = None) -> None:
-
+    def __init__(self, annotation_file: Optional[PathLikeOrStr] = None) -> None:
         self.dataset: JsonDict = {}
         self.anns: Dict[int, JsonDict] = {}
         self.cats: Dict[int, JsonDict] = {}
@@ -278,8 +365,14 @@ class CocoParser:
         """
         Print information about the annotation file.
         """
+        rows = []
         for key, value in self.dataset["info"].items():
-            print(f"{key}: {value}")
+            row = [key, value]
+            rows.append(row)
+
+        header = ["key", "value"]
+        table = tabulate(rows, headers=header, tablefmt="fancy_grid", stralign="left", numalign="left")
+        print(colored(table, "cyan"))
 
     def get_ann_ids(
         self,
@@ -289,14 +382,16 @@ class CocoParser:
         is_crowd: Optional[bool] = None,
     ) -> Sequence[int]:
         """
-        Get ann ids that satisfy given filter conditions. default skips that filter
+        Get annotation ids that satisfy given filter conditions. default skips that filter
 
-        :param img_ids: get anns for given imgs
-        :param cat_ids: get anns for given cats
-        :param area_range: get anns for given area range (e.g. [0 inf])
-        :param is_crowd: get anns for given crowd label (False or True)
+        Args:
+            img_ids: get anns for given imgs
+            cat_ids: get anns for given cats
+            area_range: get anns for given area range (e.g. [0 inf])
+            is_crowd: get anns for given crowd label (False or True)
 
-        :return: ids: integer array of ann ids
+        Returns:
+            ids: integer array of ann ids
         """
 
         if img_ids is None:
@@ -334,13 +429,15 @@ class CocoParser:
         category_ids: Optional[Union[int, Sequence[int]]] = None,
     ) -> Sequence[int]:
         """
-        Filtering parameters. default skips that filter.
+        Filtering parameters. Default does not filter anything.
 
-        :param category_names: get cats for given cat names
-        :param super_category_names: get cats for given super category names
-        :param category_ids: get cats for given cat ids
+        Args:
+            category_names: get cats for given cat names
+            super_category_names: get cats for given super category names
+            category_ids: get cats for given cat ids
 
-        :return: ids: integer array of cat ids
+        Returns:
+            ids: integer array of cat ids
         """
 
         if category_names is None:
@@ -372,12 +469,14 @@ class CocoParser:
         self, img_ids: Optional[Union[int, Sequence[int]]] = None, cat_ids: Optional[Union[int, Sequence[int]]] = None
     ) -> Sequence[int]:
         """
-        Get img ids that satisfy given filter conditions.
+        Get image ids that satisfy given filter conditions.
 
-        :param img_ids: get imgs for given ids
-        :param cat_ids: get imgs with all given cats
+        Args:
+            img_ids: get imgs for given ids
+            cat_ids: get imgs with all given cats
 
-        :return: ids: integer array of img ids
+        Returns:
+            ids: integer array of img ids
         """
 
         if img_ids is None:
@@ -403,9 +502,11 @@ class CocoParser:
         """
         Load anns with the specified ids.
 
-        :param ids: integer ids specifying anns
+        Args:
+            ids: integer ids specifying anns
 
-        :return: anns: loaded ann objects
+        Returns:
+            anns: loaded ann objects
         """
         if ids is None:
             ids = []
@@ -417,9 +518,11 @@ class CocoParser:
         """
         Load cats with the specified ids.
 
-        :param ids: integer ids specifying cats
+        Args:
+            ids: integer ids specifying cats
 
-        :return: cats: loaded cat objects
+        Returns:
+            cats: loaded cat objects
         """
         if ids is None:
             ids = []
@@ -431,9 +534,11 @@ class CocoParser:
         """
         Load anns with the specified ids.
 
-        :param ids: integer ids specifying img
+        Args:
+            ids: integer ids specifying img
 
-        :return: imgs: loaded img objects
+        Returns:
+            imgs: loaded img objects
         """
         if ids is None:
             ids = []
@@ -444,31 +549,34 @@ class CocoParser:
 
 class SerializerCoco:
     """
-    Class for serializing annotation files in Coco format. Coco comes in JSON format which is a priori not
+    Class for serializing annotation files in COCO format. COCO comes in `JSON` format which is a priori not
     serialized. This class implements only the very basic methods to generate a dataflow. It wraps the coco class
-    from pycocotools and assembles annotations that belong to the image. Note, that the conversion into the core
-    `Image` has to be done by yourself.
+    from `pycocotools` and assembles annotations that belong to the image.
+
+    Note:
+        Conversion into the core `Image` has to be done by yourself.
+
+    Example:
+        ```python
+        df = SerializerCoco.load("path/to/annotations.json")
+        df.reset_state()
+        for dp in df:
+            # {'image':{'id',...},'annotations':[{'id':…,'bbox':...}]}
+        ```
+
     """
 
     @staticmethod
-    def load(path: Pathlike, max_datapoints: Optional[int] = None) -> DataFlow:
+    def load(path: PathLikeOrStr, max_datapoints: Optional[int] = None) -> DataFlow:
         """
-        Loads a .json file and generates a dataflow.
+        Loads a `.json` file and generates a dataflow.
 
-        **Example:**
+        Args:
+            max_datapoints: Will stop the iteration once max_datapoints have been streamed.
+            path: a path to a .json file.
 
-                {'images':[img1,img2,...], 'annotations':[ann1,ann2,...],...}
-
-            it will generate a dataflow with datapoints
-
-
-                {'image':{'id',...},'annotations':[{'id':…,'bbox':...}]}
-
-            for each single image id.
-
-        :param max_datapoints: Will stop the iteration once max_datapoints have been streamed.
-        :param path: a path to a .json file.
-        :return: dataflow to iterate from
+        Returns:
+            dataflow to iterate from
         """
         if not os.path.isfile(path):
             raise FileNotFoundError(path)
@@ -484,7 +592,7 @@ class SerializerCoco:
             with get_tqdm(total=len(imgs)) as status_bar:
                 for img in imgs:
                     img["annotations"] = coco.img_to_anns[img["id"]]
-                    status_bar.update()  # type: ignore
+                    status_bar.update()
 
         df = CustomDataFromList(imgs, max_datapoints=max_datapoints)
         return df
@@ -494,37 +602,42 @@ class SerializerCoco:
         """
         Not implemented
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class SerializerPdfDoc:
     """
     Serialize a pdf document with an arbitrary number of pages.
 
-    **Example:**
-
-            df = SerializerPdfDoc.load("path/to/document.pdf")
+    Example:
+        ```python
+        df = SerializerPdfDoc.load("path/to/document.pdf")
 
         will yield datapoints:
 
-            {"path": "path/to/document.pdf", "file_name" document_page_1.pdf, "pdf_bytes": b"some-bytes"}
+        {"path": "path/to/document.pdf", "file_name" document_page_1.pdf, "pdf_bytes": b"some-bytes"}
+        ```
+
     """
 
     @staticmethod
-    def load(path: Pathlike, max_datapoints: Optional[int] = None) -> DataFlow:
+    def load(path: PathLikeOrStr, max_datapoints: Optional[int] = None) -> DataFlow:
         """
         Loads the document page wise and returns a dataflow accordingly.
 
-        :param path: Path to the pdf document.
-        :param max_datapoints: The maximum number of pages to stream.
-        :return: A dict with structure {"path":... ,"file_name": ..., "pdf_bytes": ...}. The file name is a
-                 concatenation of the physical file name and the current page number.
+        Args:
+            path: Path to the pdf document.
+            max_datapoints: The maximum number of pages to stream.
+
+        Returns:
+            A dict with structure `{"path":... ,"file_name": ..., "pdf_bytes": ...}`. The file name is a
+            concatenation of the physical file name and the current page number.
         """
 
         file_name = os.path.split(path)[1]
         prefix, suffix = os.path.splitext(file_name)
         df: DataFlow
-        df = CustomDataFromIterable(PDFStreamer(path=path), max_datapoints=max_datapoints)
+        df = CustomDataFromIterable(PDFStreamer(path_or_bytes=path), max_datapoints=max_datapoints)
         df = MapData(
             df,
             lambda dp: {
@@ -538,14 +651,16 @@ class SerializerPdfDoc:
         return df
 
     @staticmethod
-    def save(path: Pathlike) -> None:
+    def save(path: PathLikeOrStr) -> None:
         """
         Not implemented
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @staticmethod
-    def split(path: Pathlike, path_target: Optional[Pathlike] = None, max_datapoint: Optional[int] = None) -> None:
+    def split(
+        path: PathLikeOrStr, path_target: Optional[PathLikeOrStr] = None, max_datapoint: Optional[int] = None
+    ) -> None:
         """
         Split a document into single pages.
         """
