@@ -28,7 +28,7 @@ Module for Publaynet dataset. Place the dataset as follows
     ├── train.json
     ├── val.json
 """
-
+from __future__ import annotations
 
 from typing import Mapping, Union
 
@@ -43,6 +43,7 @@ from ..info import DatasetCategories, DatasetInfo
 from ..registry import dataset_registry
 
 _NAME = "publaynet"
+_SHORT_DESCRIPTION = "PubLayNet is a dataset for document layout analysis."
 _DESCRIPTION = (
     "PubLayNet is a dataset for document layout analysis. It contains images of research papers and "
     "articles \n"
@@ -61,30 +62,38 @@ _URL = (
     "publaynet.tar.gz?_ga=2.23017467.1796315263.1628754613-1173244232.1625045842"
 )
 _SPLITS: Mapping[str, str] = {"train": "train", "val": "val"}
-_TYPE = DatasetType.object_detection
+_TYPE = DatasetType.OBJECT_DETECTION
 
 _LOCATION = "publaynet"
 
 _ANNOTATION_FILES: Mapping[str, str] = {"train": "train.json", "val": "val.json"}
-_INIT_CATEGORIES = [LayoutType.text, LayoutType.title, LayoutType.list, LayoutType.table, LayoutType.figure]
+_INIT_CATEGORIES = [LayoutType.TEXT, LayoutType.TITLE, LayoutType.LIST, LayoutType.TABLE, LayoutType.FIGURE]
 
 
 @dataset_registry.register("publaynet")
 class Publaynet(_BuiltInDataset):
     """
-    Publaynet
+    `Publaynet`
     """
 
     _name = _NAME
 
     @classmethod
     def _info(cls) -> DatasetInfo:
-        return DatasetInfo(name=_NAME, description=_DESCRIPTION, license=_LICENSE, url=_URL, splits=_SPLITS, type=_TYPE)
+        return DatasetInfo(
+            name=_NAME,
+            short_description=_SHORT_DESCRIPTION,
+            description=_DESCRIPTION,
+            license=_LICENSE,
+            url=_URL,
+            splits=_SPLITS,
+            type=_TYPE,
+        )
 
     def _categories(self) -> DatasetCategories:
         return DatasetCategories(init_categories=_INIT_CATEGORIES)
 
-    def _builder(self) -> "PublaynetBuilder":
+    def _builder(self) -> PublaynetBuilder:
         return PublaynetBuilder(location=_LOCATION, annotation_files=_ANNOTATION_FILES)
 
 
@@ -98,15 +107,15 @@ class PublaynetBuilder(DataFlowBaseBuilder):
         Returns a dataflow from which you can stream datapoints of images. The following arguments affect the returns
         of the dataflow:
 
-        `split:` Split of the dataset. Can be `train`,`val` or `test`. Default: `val`
+        Args:
+            kwargs:
+                (split) Split of the dataset. Can be `train`, `val` or `test`. Default: `val`
+                (max_datapoints) Will stop iterating after `max_datapoints`. Default: `None`
+                (load_image) Will load the image for each datapoint. Default: `False`
+                (fake_score) Will add a fake score so that annotations look like predictions. Default: `False`
 
-        `max_datapoints:` Will stop iterating after max_datapoints. Default: `None`
-
-        `load_image:` Will load the image for each datapoint.  Default: `False`
-
-        `fake_score:` Will add a fake score so that annotations look like predictions
-
-        :return: dataflow
+        Returns:
+            Dataflow
         """
         split = str(kwargs.get("split", "val"))
         max_datapoints = kwargs.get("max_datapoints")
@@ -120,7 +129,7 @@ class PublaynetBuilder(DataFlowBaseBuilder):
         df = SerializerCoco.load(path, max_datapoints=max_datapoints)
 
         # Map
-        df = MapDataComponent(df, lambda dp: self.get_workdir() / self.get_split(split) / dp, "file_name")
+        df = MapDataComponent(df, lambda dp: (self.get_workdir() / self.get_split(split) / dp).as_posix(), "file_name")
         coco_mapper = coco_to_image(  # pylint: disable=E1120  # 259
             self.categories.get_categories(init=True),
             load_image,
