@@ -19,36 +19,35 @@
 """
 Module for `Evaluator`
 """
-
-__all__ = ["Evaluator"]
+from __future__ import annotations
 
 from copy import deepcopy
-from typing import Any, Dict, List, Literal, Mapping, Optional, Type, Union, overload
+from typing import Any, Generator, Literal, Mapping, Optional, Type, Union, overload
 
 import numpy as np
+from lazy_imports import try_import
 
 from ..dataflow import CacheData, DataFlow, DataFromList, MapData
 from ..datapoint.image import Image
 from ..datasets.base import DatasetBase
 from ..mapper.cats import filter_cat, remove_cats
+from ..mapper.d2struct import to_wandb_image
 from ..mapper.misc import maybe_load_image, maybe_remove_image, maybe_remove_image_from_category
-from ..pipe.base import LanguageModelPipelineComponent, PredictorPipelineComponent
+from ..pipe.base import PipelineComponent
 from ..pipe.common import PageParsingService
 from ..pipe.concurrency import MultiThreadPipelineComponent
 from ..pipe.doctectionpipe import DoctectionPipe
-from ..utils.detection_types import ImageType
-from ..utils.file_utils import detectron2_available, wandb_available
-from ..utils.logger import logger
-from ..utils.settings import DatasetType, LayoutType, TypeOrStr
+from ..utils.logger import LoggingRecord, logger
+from ..utils.settings import DatasetType, LayoutType, TypeOrStr, get_type
+from ..utils.types import PixelValues
 from ..utils.viz import interactive_imshow
 from .base import MetricBase
 
-if wandb_available():
+with try_import() as wb_import_guard:
     import wandb  # pylint:disable=W0611
     from wandb import Artifact, Table
 
-if wandb_available() and detectron2_available():
-    from ..mapper.d2struct import to_wandb_image
+__all__ = ["Evaluator"]
 
 
 class Evaluator:
@@ -66,42 +65,45 @@ class Evaluator:
     erasing process and after that passing the predictor. Predicted and gt datapoints will be converted into the
     required metric input format and dumped into lists. Both lists will be passed to `MetricBase.get_distance`.
 
-    **Example:**
-
+    Note:
         You can evaluate the predictor on a subset of categories by filtering the ground truth dataset. When using
         the coco metric all predicted objects that are not in the set of filtered objects will be not taken into
         account.
 
-            publaynet = get_dataset("publaynet")
-            publaynet.dataflow.categories.filter_categories(categories=["TEXT","TITLE"])
-            coco_metric = metric_registry.get("coco")
-            profile = ModelCatalog.get_profile("layout/d2_model_0829999_layout_inf_only.pt")
-            path_weights = ModelCatalog.get_full_path_weights("layout/d2_model_0829999_layout_inf_only.pt")
-            path_config_yaml= ModelCatalog.get_full_path_configs("layout/d2_model_0829999_layout_inf_only.pt")
+    Example:
+        ```python
+        publaynet = get_dataset("publaynet")
+        publaynet.dataflow.categories.filter_categories(categories=["TEXT","TITLE"])
+        coco_metric = metric_registry.get("coco")
+        profile = ModelCatalog.get_profile("layout/d2_model_0829999_layout_inf_only.pt")
+        path_weights = ModelCatalog.get_full_path_weights("layout/d2_model_0829999_layout_inf_only.pt")
+        path_config_yaml= ModelCatalog.get_full_path_configs("layout/d2_model_0829999_layout_inf_only.pt")
 
-            layout_detector = D2FrcnnDetector(path_config_yaml, path_weights, profile.categories)
-            layout_service = ImageLayoutService(layout_detector)
-            evaluator = Evaluator(publaynet, layout_service, coco_metric)
+        layout_detector = D2FrcnnDetector(path_config_yaml, path_weights, profile.categories)
+        layout_service = ImageLayoutService(layout_detector)
+        evaluator = Evaluator(publaynet, layout_service, coco_metric)
 
-            output = evaluator.run(max_datapoints=10)
+        output = evaluator.run(max_datapoints=10)
+        ```
 
-    For another example check the script in :ref:`Evaluation of table recognition`
+    For another example check the script in `Evaluation` of table recognition`
     """
 
     def __init__(
         self,
         dataset: DatasetBase,
-        component_or_pipeline: Union[PredictorPipelineComponent, LanguageModelPipelineComponent, DoctectionPipe],
+        component_or_pipeline: Union[PipelineComponent, DoctectionPipe],
         metric: Union[Type[MetricBase], MetricBase],
         num_threads: int = 2,
-        run: Optional["wandb.sdk.wandb_run.Run"] = None,
+        run: Optional[wandb.sdk.wandb_run.Run] = None,
     ) -> None:
         """
         Evaluating a pipeline component on a dataset with a given metric.
 
-        :param dataset: dataset
-        :param component_or_pipeline: A pipeline component with predictor and annotation factory.
-        :param metric: metric
+        Args:
+            dataset: dataset
+            component_or_pipeline: A pipeline component with predictor and annotation factory.
+            metric: metric
         """
 
         self.dataset = dataset
@@ -109,12 +111,14 @@ class Evaluator:
         self.pipe: Optional[DoctectionPipe] = None
 
         # when passing a component, we will process prediction on num_threads
-        if isinstance(component_or_pipeline, (PredictorPipelineComponent, LanguageModelPipelineComponent)):
+        if isinstance(component_or_pipeline, PipelineComponent):
             logger.info(
-                "Building multi threading pipeline component to increase prediction throughput. Using %i threads",
-                num_threads,
+                LoggingRecord(
+                    f"Building multi threading pipeline component to increase prediction throughput. "
+                    f"Using {num_threads} threads"
+                )
             )
-            pipeline_components: List[Union[PredictorPipelineComponent, LanguageModelPipelineComponent]] = []
+            pipeline_components: list[PipelineComponent] = []
 
             for _ in range(num_threads - 1):
                 copy_pipe_component = component_or_pipeline.clone()
@@ -138,33 +142,65 @@ class Evaluator:
 
         self.wandb_table_agent: Optional[WandbTableAgent]
         if run is not None:
-            self.wandb_table_agent = WandbTableAgent(
-                run, self.dataset.dataset_info.name, 50, self.dataset.dataflow.categories.get_categories(filtered=True)
-            )
+            if self.dataset.dataset_info.type == DatasetType.OBJECT_DETECTION:
+                self.wandb_table_agent = WandbTableAgent(
+                    run,
+                    self.dataset.dataset_info.name,
+                    50,
+                    self.dataset.dataflow.categories.get_categories(filtered=True),
+                )
+            elif self.dataset.dataset_info.type == DatasetType.TOKEN_CLASSIFICATION:
+                if hasattr(self.metric, "sub_cats"):
+                    sub_cat_key, sub_cat_val_list = list(self.metric.sub_cats.items())[0]
+                    sub_cat_val = sub_cat_val_list[0]
+                    sub_cats = {sub_cat_key: sub_cat_val}
+                    self.wandb_table_agent = WandbTableAgent(
+                        run,
+                        self.dataset.dataset_info.name,
+                        50,
+                        self.dataset.dataflow.categories.get_categories(filtered=True),
+                        self.dataset.dataflow.categories.get_sub_categories(
+                            categories=sub_cat_key,
+                            sub_categories=sub_cats,
+                            keys=False,
+                            values_as_dict=True,
+                            name_as_key=False,
+                        )[sub_cat_key][sub_cat_val],
+                        sub_cats,
+                    )
+                else:
+                    raise AttributeError(
+                        "metric has no attribute sub_cats and cannot be used for token classification datasets"
+                    )
+            else:
+                raise NotImplementedError()
+
         else:
             self.wandb_table_agent = None
 
     @overload
     def run(
         self, output_as_dict: Literal[False] = False, **dataflow_build_kwargs: Union[str, int]
-    ) -> List[Dict[str, float]]:
+    ) -> list[dict[str, float]]:
         ...
 
     @overload
-    def run(self, output_as_dict: Literal[True], **dataflow_build_kwargs: Union[str, int]) -> Dict[str, float]:
+    def run(self, output_as_dict: Literal[True], **dataflow_build_kwargs: Union[str, int]) -> dict[str, float]:
         ...
 
     def run(
         self, output_as_dict: bool = False, **dataflow_build_kwargs: Union[str, int]
-    ) -> Union[List[Dict[str, float]], Dict[str, float]]:
+    ) -> Union[list[dict[str, float]], dict[str, float]]:
         """
         Start evaluation process and return the results.
 
-        :param output_as_dict: Return result in a list or dict.
-        :param dataflow_build_kwargs: Pass the necessary arguments in order to build the dataflow, e.g. "split",
-                                      "build_mode", "max_datapoints" etc.
+        Args:
+            output_as_dict: Return result in a list or dict.
+            dataflow_build_kwargs: Pass the necessary arguments in order to build the dataflow, e.g. `split`,
+                                  `build_mode`, `max_datapoints` etc.
 
-        :return: dict with metric results.
+        Returns:
+            dict with metric results.
         """
 
         df_gt = self.dataset.dataflow.build(**dataflow_build_kwargs)
@@ -174,7 +210,7 @@ class Evaluator:
         df_pr = self._clean_up_predict_dataflow_annotations(df_pr)
         df_pr = self._run_pipe_or_component(df_pr)
 
-        logger.info("Starting evaluation...")
+        logger.info(LoggingRecord("Starting evaluation..."))
         result = self.metric.get_distance(df_gt, df_pr, self.dataset.dataflow.categories)
         self.metric.print_result()
 
@@ -192,7 +228,7 @@ class Evaluator:
     def _run_pipe_or_component(self, df_pr: DataFlow) -> DataFlow:
         if self.pipe_component:
             self.pipe_component.put_task(df_pr)
-            logger.info("Predicting objects...")
+            logger.info(LoggingRecord("Predicting objects..."))
             df_pr_list = self.pipe_component.start()
             if self.wandb_table_agent is not None:
                 df_pr_list = [self.wandb_table_agent.dump(dp) for dp in df_pr_list]
@@ -215,11 +251,11 @@ class Evaluator:
         possible_cats_in_datapoint = self.dataset.dataflow.categories.get_categories(as_dict=False, filtered=True)
 
         # clean-up procedure depends on the dataset type
-        if self.dataset.dataset_info.type == DatasetType.object_detection:
+        if self.dataset.dataset_info.type == DatasetType.OBJECT_DETECTION:
             # we keep all image annotations that will not be generated through processing
-            anns_to_keep = {ann for ann in possible_cats_in_datapoint if ann not in meta_anns["image_annotations"]}
-            sub_cats_to_remove = meta_anns["sub_categories"]
-            relationships_to_remove = meta_anns["relationships"]
+            anns_to_keep = {ann for ann in possible_cats_in_datapoint if ann not in meta_anns.image_annotations}
+            sub_cats_to_remove = meta_anns.sub_categories
+            relationships_to_remove = meta_anns.relationships
             # removing annotations takes place in three steps: First we remove all image annotations. Then, with all
             # remaining image annotations we check, if the image attribute (with Image instance !) is not empty and
             # remove it as well, if necessary. In the last step we remove all sub categories and relationships, if
@@ -231,45 +267,54 @@ class Evaluator:
                 remove_cats(sub_categories=sub_cats_to_remove, relationships=relationships_to_remove),
             )
 
-        elif self.dataset.dataset_info.type == DatasetType.sequence_classification:
-            summary_sub_cats_to_remove = meta_anns["summaries"]
+        elif self.dataset.dataset_info.type == DatasetType.SEQUENCE_CLASSIFICATION:
+            summary_sub_cats_to_remove = meta_anns.summaries
             df_pr = MapData(df_pr, remove_cats(summary_sub_categories=summary_sub_cats_to_remove))
 
-        elif self.dataset.dataset_info.type == DatasetType.token_classification:
-            sub_cats_to_remove = meta_anns["sub_categories"]
+        elif self.dataset.dataset_info.type == DatasetType.TOKEN_CLASSIFICATION:
+            sub_cats_to_remove = meta_anns.sub_categories
             df_pr = MapData(df_pr, remove_cats(sub_categories=sub_cats_to_remove))
         else:
-            raise NotImplementedError
+            raise NotImplementedError()
 
         return df_pr
 
-    def compare(self, interactive: bool = False, **dataflow_build_kwargs: Union[str, int]) -> Optional[ImageType]:
+    def compare(self, interactive: bool = False, **kwargs: Union[str, int]) -> Generator[PixelValues, None, None]:
         """
         Visualize ground truth and prediction datapoint. Given a dataflow config it will run predictions per sample
         and concat the prediction image (with predicted bounding boxes) with ground truth image.
 
-        :param interactive: If set to True will open an interactive image, otherwise it will return a numpy array that
-                            can be displayed differently (e.g. matplotlib). Note that, if the interactive mode is being
-                            used, more than one sample can be iteratively be displayed.
-        :param dataflow_build_kwargs: Dataflow configs for displaying specific image splits
-        :return: Image as numpy array
+        Args:
+            interactive: If set to True will open an interactive image, otherwise it will return a `np.array` that
+                        can be displayed differently (e.g. `matplotlib`). Note that, if the interactive mode is being
+                        used, more than one sample can be iteratively be displayed.
+            kwargs: Dataflow configs for displaying specific image splits and visualisation configs:
+                   `show_tables`, `show_layouts`, `show_table_structure`, `show_words`
+
+        Returns:
+            Image as `np.array`
         """
-        df_gt = self.dataset.dataflow.build(**dataflow_build_kwargs)
-        df_pr = self.dataset.dataflow.build(**dataflow_build_kwargs)
+
+        show_tables = kwargs.pop("show_tables", True)
+        show_layouts = kwargs.pop("show_layouts", True)
+        show_table_structure = kwargs.pop("show_table_structure", True)
+        show_words = kwargs.pop("show_words", False)
+        show_token_class = kwargs.pop("show_token_class", True)
+        ignore_default_token_class = kwargs.pop("ignore_default_token_class", False)
+        floating_text_block_categories = kwargs.pop("floating_text_block_categories", None)
+        include_residual_text_containers = kwargs.pop("include_residual_Text_containers", True)
+
+        df_gt = self.dataset.dataflow.build(**kwargs)
+        df_pr = self.dataset.dataflow.build(**kwargs)
         df_gt = MapData(df_gt, maybe_load_image)
         df_pr = MapData(df_pr, maybe_load_image)
         df_pr = MapData(df_pr, deepcopy)
         df_pr = self._clean_up_predict_dataflow_annotations(df_pr)
 
         page_parsing_component = PageParsingService(
-            text_container=LayoutType.word,
-            top_level_text_block_names=[
-                LayoutType.title,
-                LayoutType.text,
-                LayoutType.list,
-                LayoutType.table,
-                LayoutType.figure,
-            ],
+            text_container=LayoutType.WORD,
+            floating_text_block_categories=floating_text_block_categories,  # type: ignore
+            include_residual_text_container=bool(include_residual_text_containers),
         )
         df_gt = page_parsing_component.predict_dataflow(df_gt)
 
@@ -286,52 +331,79 @@ class Evaluator:
         df_pr.reset_state()
         df_gt.reset_state()
         for dp_gt, dp_pred in zip(df_gt, df_pr):
-            img_gt, img_pred = dp_gt.viz(), dp_pred.viz()
+            img_gt, img_pred = dp_gt.viz(
+                show_tables=show_tables,
+                show_layouts=show_layouts,
+                show_table_structure=show_table_structure,
+                show_words=show_words,
+                show_token_class=show_token_class,
+                ignore_default_token_class=ignore_default_token_class,
+            ), dp_pred.viz(
+                show_tables=show_tables,
+                show_layouts=show_layouts,
+                show_table_structure=show_table_structure,
+                show_words=show_words,
+                show_token_class=show_token_class,
+                ignore_default_token_class=ignore_default_token_class,
+            )
             img_concat = np.concatenate((img_gt, img_pred), axis=1)
             if interactive:
                 interactive_imshow(img_concat)
             else:
-                return img_concat
-        return None
+                yield img_concat
 
 
 class WandbTableAgent:
     """
     A class that creates a W&B table of sample predictions and sends them to the W&B server.
 
+    Example:
+        ```python
         df ... # some dataflow
         agent = WandbTableAgent(myrun,"MY_DATASET",50,{"1":"FOO"})
         for dp in df:
             agent.dump(dp)
 
         agent.log()
-
+        ```
     """
 
     def __init__(
         self,
-        wandb_run: "wandb.sdk.wandb_run.Run",
+        wandb_run: wandb.sdk.wandb_run.Run,
         dataset_name: str,
         num_samples: int,
-        categories: Mapping[str, TypeOrStr],
+        categories: Mapping[int, TypeOrStr],
+        sub_categories: Optional[Mapping[int, TypeOrStr]] = None,
+        cat_to_sub_cat: Optional[Mapping[TypeOrStr, TypeOrStr]] = None,
     ):
         """
-        :param wandb_run: An `wandb.run` instance for tracking. Use `run=wandb.init(project=project, config=config,
-                          **kwargs)` to generate a `run`.
-        :param dataset_name: name for tracking
-        :param num_samples: When dumping images to a table it will stop adding samples after `num_samples` instances
-        :param categories: dict of all possible categories
+        Args:
+            wandb_run: An `wandb.run` instance for tracking. Use `run=wandb.init(project=project, config=config,
+                      **kwargs)` to generate a `run`.
+            dataset_name: name for tracking
+            num_samples: When dumping images to a table it will stop adding samples after `num_samples` instances
+            categories: dict of all possible categories
+            sub_categories: dict of sub categories. If provided, these categories will define the classes for the
+                            table
+            cat_to_sub_cat: dict of category to sub category keys. Suppose your category `foo` has a sub category
+                           defined by the key `sub_foo`. The range sub category values must then be given by
+                           `sub_categories` and to extract the sub category values one must pass `{"foo": "sub_foo"}`
         """
 
         self.dataset_name = dataset_name
         self.num_samples = num_samples
         self.categories = categories
+        self.sub_categories = sub_categories
+        if cat_to_sub_cat is None:
+            cat_to_sub_cat = {}
+        self.cat_to_sub_cat = {get_type(cat): get_type(sub_cat) for cat, sub_cat in cat_to_sub_cat.items()}
         self._run = wandb_run
         self._counter = 0
 
         # Table logging utils
-        self._table_cols: List[str] = ["file_name", "image"]
-        self._table_rows: List[Any] = []
+        self._table_cols: list[str] = ["file_name", "image"]
+        self._table_rows: list[Any] = []
         self._table_ref = None
 
     def dump(self, dp: Image) -> Image:
@@ -339,12 +411,17 @@ class WandbTableAgent:
         Dump image to a table. Add this while iterating over samples. After `num_samples` it will stop appending samples
         to the table
 
-        :param dp: datapoint image
-        :return: same as input
+        Args:
+            dp: `Image` instance
+
+        Returns:
+            `Image` instance
         """
         if self.num_samples > self._counter:
             dp = maybe_load_image(dp)
-            self._table_rows.append(to_wandb_image(self.categories)(dp))  # pylint: disable=E1102
+            self._table_rows.append(
+                to_wandb_image(self.categories, self.sub_categories, self.cat_to_sub_cat)(dp)  # pylint: disable=E1102
+            )
             dp = maybe_remove_image(dp)
             self._counter += 1
         return dp
@@ -356,11 +433,12 @@ class WandbTableAgent:
         self._table_rows = []
         self._counter = 0
 
-    def _build_table(self) -> "Table":
+    def _build_table(self) -> Table:
         """
-        Builds wandb.Table object for logging evaluation
+        Builds `wandb.Table` instance for logging evaluation
 
-        returns: Table object to log evaluation
+        Returns:
+            Table object to log evaluation
         """
         return Table(columns=self._table_cols, data=self._table_rows)
 
@@ -382,4 +460,4 @@ class WandbTableAgent:
         eval_art.add(self._build_table(), self.dataset_name)
         self._run.use_artifact(eval_art)
         eval_art.wait()
-        self._table_ref = eval_art.get(self.dataset_name).data  # type:ignore
+        self._table_ref = eval_art.get(self.dataset_name).data  # type: ignore
