@@ -16,14 +16,15 @@
 # limitations under the License.
 
 """
-Module for mapping annotations to and from prodigy data structure
+Module for mapping annotations to and from prodigy data structure.
 """
 
 import os
-from typing import Mapping, Optional
+from typing import Mapping, Optional, Sequence
 
 from ..datapoint import BoundingBox, Image, ImageAnnotation
-from ..utils.detection_types import JsonDict, Pathlike
+from ..utils.settings import ObjectTypes, get_type
+from ..utils.types import JsonDict, PathLikeOrStr
 from .maputils import MappingContextManager, curry, maybe_get_fake_score
 
 _PRODIGY_IMAGE_PREFIX = "data:image/png;base64,"
@@ -32,27 +33,32 @@ _PRODIGY_IMAGE_PREFIX = "data:image/png;base64,"
 @curry
 def prodigy_to_image(
     dp: JsonDict,
-    categories_name_as_key: Mapping[str, str],
+    categories_name_as_key: Mapping[ObjectTypes, int],
     load_image: bool,
     fake_score: bool,
-    path_reference_ds: Optional[Pathlike] = None,
+    path_reference_ds: Optional[PathLikeOrStr] = None,
     accept_only_answer: bool = False,
     category_name_mapping: Optional[Mapping[str, str]] = None,
 ) -> Optional[Image]:
     """
-    Map a datapoint of annotation structure as given as from Prodigy database to an Image
-    structure.
+    Maps a datapoint of annotation structure from Prodigy database to an `Image` structure.
+
+    Args:
+        dp: A datapoint in dict structure as returned from Prodigy database.
+        categories_name_as_key: A dict of categories, e.g. `DatasetCategories.get_categories(name_as_key=True)`.
+        load_image: If `True`, it will load image to `Image.image`.
+        fake_score: If `dp` does not contain a score, a fake score with uniform random variables in (0,1) will be added.
+        path_reference_ds: A path to a reference-dataset. It must point to the basedir where the file of the datapoint
+                           can be found.
+        accept_only_answer: Filter every datapoint that has the answer `reject` or `ignore`.
+        category_name_mapping: Map incoming category names, e.g. `{"source_name":"target_name"}`.
+
+    Returns:
+        `Image`
+
+    Note:
+        If `accept_only_answer` is `True`, only datapoints with the answer `accept` will be processed.
 
-    :param dp: A datapoint in dict structure as returned from Prodigy database
-    :param categories_name_as_key: A dict of categories, e.g. DatasetCategories.get_categories(name_as_key=True)
-    :param load_image: If 'True' it will load image to attr:`Image.image`
-    :param fake_score: If dp does not contain a score, a fake score with uniform random variables in (0,1)
-                       will be added.
-    :param path_reference_ds: A path to a reference-dataset. It must point to the basedir where the file
-                              of the datapoint can be found.
-    :param accept_only_answer: Filter every datapoint that has the answer 'reject' or 'ignore'.
-    :param category_name_mapping: Map incoming category names, e.g. {"source_name":"target_name"}
-    :return: Image
     """
 
     if accept_only_answer and dp.get("answer") != "accept":
@@ -127,12 +133,12 @@ def prodigy_to_image(
             else:
                 label = span["label"]
             if not isinstance(label, str):
-                raise ValueError("label could not assigned to be a string")
+                raise TypeError("label must be a string")
 
             annotation = ImageAnnotation(
                 category_name=label,
                 bounding_box=bbox,
-                category_id=categories_name_as_key[label],
+                category_id=categories_name_as_key[get_type(label)],
                 score=score,
                 external_id=external_id,
             )
@@ -143,16 +149,26 @@ def prodigy_to_image(
     return image
 
 
-def image_to_prodigy(dp: Image) -> JsonDict:
+@curry
+def image_to_prodigy(dp: Image, category_names: Optional[Sequence[ObjectTypes]] = None) -> JsonDict:
     """
-    The mapper to transform the normalized image representation of datasets into the format
-    for visualising the annotation components in Prodigy.
+    Transforms the normalized image representation of datasets into the format for visualizing the annotation
+    components in Prodigy.
+
+    Args:
+        dp: An `Image`.
+        category_names: A list of category names to filter the annotations.
+
+    Returns:
+        A dictionary with compulsory keys: `text` and `spans`.
+
+    Example:
+        ```python
+        image_to_prodigy(image_instance)
+        ```
 
-    :param dp: An image
-    :return: A dictionary with compulsory keys: "text" and "spans"
     """
 
-    assert isinstance(dp, Image), f"datapoints must be of type Image, is of type {type(dp)}"
     output: JsonDict = {}
     img_str = dp.get_image().to_b64()
     if img_str is not None:
@@ -161,17 +177,8 @@ def image_to_prodigy(dp: Image) -> JsonDict:
     output["image_id"] = dp.image_id
 
     spans = []
-    for ann in dp.get_annotation_iter():
-        if ann.bounding_box is None:
-            raise ValueError("ann.bounding_box cannot be None")
-        box: JsonDict = {"label": ann.category_name, "annotation_id": ann.annotation_id}
-        if ann.score is not None:
-            box["score"] = float(round(ann.score, 3))
-        box["type"] = "rect"
-        if ann.image is not None:
-            bounding_box = ann.image.get_embedding(dp.image_id)
-        else:
-            bounding_box = ann.bounding_box
+    for ann in dp.get_annotation(category_names=category_names):
+        bounding_box = ann.get_bounding_box(dp.image_id)
         if not bounding_box.absolute_coords:
             bounding_box = bounding_box.transform(dp.width, dp.height, absolute_coords=True)
         boxes = [
@@ -181,9 +188,15 @@ def image_to_prodigy(dp: Image) -> JsonDict:
             [bounding_box.lrx, bounding_box.uly],
         ]
 
-        box["points"] = [list(map(float, box)) for box in boxes]
+        span: JsonDict = {
+            "label": ann.category_name.value,  # type: ignore
+            "annotation_id": ann.annotation_id,
+            "type": "rect",
+            "points": boxes,
+            "score": float(round(ann.score, 3)) if ann.score is not None else None,
+        }
+        spans.append(span)
 
-        spans.append(box)
     output["spans"] = spans
     output["meta"] = {"file_name": dp.file_name}
     return output
