@@ -16,21 +16,20 @@
 # limitations under the License.
 
 """
-Module for mapping annotations in pubtabnet style structure
+Mapping annotations from Pubtabnet structure to Image structure.
 """
 import itertools
 import os
-from typing import Dict, Iterable, List, Optional, Sequence, Tuple
+from typing import Iterable, Optional, Sequence
 
 import numpy as np
 
 from ..datapoint import BoundingBox, CategoryAnnotation, ContainerAnnotation, ImageAnnotation
-from ..datapoint.annotation import SummaryAnnotation
 from ..datapoint.convert import convert_pdf_bytes_to_np_array_v2
 from ..datapoint.image import Image
-from ..utils.detection_types import JsonDict
 from ..utils.fs import load_bytes_from_pdf_file, load_image_from_file
-from ..utils.settings import CellType, LayoutType, Relationships, TableType, WordType
+from ..utils.settings import CellType, LayoutType, ObjectTypes, Relationships, SummaryType, TableType, WordType
+from ..utils.types import JsonDict, PubtabnetDict
 from ..utils.utils import is_file_extension
 from .maputils import MappingContextManager, curry, maybe_get_fake_score
 
@@ -38,7 +37,6 @@ __all__ = ["pub_to_image"]
 
 
 def _convert_boxes(dp: JsonDict, height: int) -> JsonDict:
-
     if "bbox" in dp:
         table_box_3 = height - dp["bbox"][1]
         dp["bbox"][1] = height - dp["bbox"][3]
@@ -53,16 +51,14 @@ def _convert_boxes(dp: JsonDict, height: int) -> JsonDict:
     return dp
 
 
-def _get_table_annotation(dp: JsonDict, category_id: str) -> ImageAnnotation:
-
+def _get_table_annotation(dp: JsonDict, category_id: int) -> ImageAnnotation:
     ulx, uly, lrx, lry = list(map(float, dp["bbox"]))
     bbox = BoundingBox(absolute_coords=True, ulx=ulx, uly=uly, lrx=lrx, lry=lry)
-    annotation = ImageAnnotation(category_name=LayoutType.table, bounding_box=bbox, category_id=category_id)
+    annotation = ImageAnnotation(category_name=LayoutType.TABLE, bounding_box=bbox, category_id=category_id)
     return annotation
 
 
-def _cell_token(html: Sequence[str]) -> List[List[int]]:
-
+def _cell_token(html: Sequence[str]) -> list[list[int]]:
     index_rows = [i for i, tag in enumerate(html) if tag == "<tr>"]
     index_cells = [i for i, tag in enumerate(html) if tag in ("<td>", ">")]
     index_rows_tmp = [(index_rows[i], index_rows[i + 1]) for i in range(len(index_rows) - 1)]
@@ -75,16 +71,17 @@ def _cell_token(html: Sequence[str]) -> List[List[int]]:
     return index_cells_tmp
 
 
-def _item_spans(html: Sequence[str], index_cells: Sequence[Sequence[int]], item: str) -> List[List[int]]:
-
+def _item_spans(html: Sequence[str], index_cells: Sequence[Sequence[int]], item: str) -> list[list[int]]:
     item_spans = [
         [
-            int(html[index_cell - 1].replace(item + "=", "").replace('"', ""))
-            if (item in html[index_cell - 1] and html[index_cell] == ">")
-            else (
-                int(html[index_cell - 2].replace(item + "=", "").replace('"', ""))
-                if (item in html[index_cell - 2] and html[index_cell] == ">")
-                else 1
+            (
+                int(html[index_cell - 1].replace(item + "=", "").replace('"', ""))
+                if (item in html[index_cell - 1] and html[index_cell] == ">")
+                else (
+                    int(html[index_cell - 2].replace(item + "=", "").replace('"', ""))
+                    if (item in html[index_cell - 2] and html[index_cell] == ">")
+                    else 1
+                )
             )
             for index_cell in index_cell_per_row
         ]
@@ -104,15 +101,18 @@ def _end_of_header(html: Sequence[str]) -> int:
     return 0
 
 
-def tile_table(row_spans: Sequence[Sequence[int]], col_spans: Sequence[Sequence[int]]) -> List[List[int]]:
+def tile_table(row_spans: Sequence[Sequence[int]], col_spans: Sequence[Sequence[int]]) -> list[list[int]]:
     """
-    Tiles a table according the row and column span scheme. A table can be represented as a list of list, where each
-    inner list has the same length. Each cell with a cell id can be located according to their row and column spans in
-    that scheme
+    Tiles a table according to the row and column span scheme. A table can be represented as a list of lists, where
+    each inner list has the same length. Each cell with a cell id can be located according to their row and column
+    spans in that scheme.
 
-    :param row_spans: A list of list of row spans
-    :param col_spans: A list of list of column spans
-    :return: A list of list of the tiling of the table, indicating the precise place of each cell.
+    Args:
+        row_spans: A list of lists of row spans.
+        col_spans: A list of lists of column spans.
+
+    Returns:
+        A list of lists of the tiling of the table, indicating the precise place of each cell.
     """
     number_of_cols = sum(col_spans[0])
     number_of_rows = len(col_spans)
@@ -121,7 +121,6 @@ def tile_table(row_spans: Sequence[Sequence[int]], col_spans: Sequence[Sequence[
     for row in col_spans:
         cell_id_per_row = []
         for idx, k in enumerate(itertools.count(i)):
-
             if idx < len(row):
                 i += 1
                 cell_id_per_row.append(k)
@@ -132,7 +131,7 @@ def tile_table(row_spans: Sequence[Sequence[int]], col_spans: Sequence[Sequence[
     tiling = [[-1] * number_of_cols for _ in range(number_of_rows)]  # initialize placeholders
     table = zip(cell_ids, row_spans, col_spans)
 
-    for row_id, row in enumerate(table):  # type: ignore # pylint: disable=R1702
+    for row_id, row in enumerate(table):  # type: ignore
         for cell in zip(row[0], row[1], row[2]):  # type: ignore
             cell_id = cell[0]
             row_span = cell[1]
@@ -156,45 +155,41 @@ def tile_table(row_spans: Sequence[Sequence[int]], col_spans: Sequence[Sequence[
     return tiling
 
 
-def _add_items(image: Image, item_type: str, categories_name_as_key: Dict[str, str], pubtables_like: bool) -> Image:
+def _add_items(
+    image: Image, item_type: str, categories_name_as_key: dict[ObjectTypes, int], pubtables_like: bool
+) -> Image:
+    item_number = CellType.ROW_NUMBER if item_type == LayoutType.ROW else CellType.COLUMN_NUMBER
+    item_span = CellType.ROW_SPAN if item_type == LayoutType.ROW else CellType.COLUMN_SPAN
 
-    item_number = CellType.row_number if item_type == LayoutType.row else CellType.column_number
-    item_span = CellType.row_span if item_type == LayoutType.row else CellType.column_span
+    summary_key = TableType.NUMBER_OF_ROWS if item_type == LayoutType.ROW else TableType.NUMBER_OF_COLUMNS
 
-    summary_key = TableType.number_of_rows if item_type == LayoutType.row else TableType.number_of_columns
+    category_item = image.summary.get_sub_category(summary_key)
+    number_of_items = category_item.category_id
 
-    number_of_items = 0
-
-    if image.summary is not None:
-        category_item = image.summary.get_sub_category(summary_key)
-        number_of_items = int(category_item.category_id)
-
-    cells = image.get_annotation(category_names=LayoutType.cell)
+    cells = image.get_annotation(category_names=LayoutType.CELL)
     table: ImageAnnotation
 
     for item_num in range(1, number_of_items + 1):
         cell_item = list(
-            filter(
-                lambda x: x.get_sub_category(item_number).category_id == str(item_num), cells  # pylint: disable=W0640
-            )
+            filter(lambda x: x.get_sub_category(item_number).category_id == item_num, cells)  # pylint: disable=W0640
         )
-        cell_item = list(filter(lambda x: x.get_sub_category(item_span).category_id == "1", cell_item))
+        cell_item = list(filter(lambda x: x.get_sub_category(item_span).category_id == 1, cell_item))
         if cell_item:
-            ulx = min([cell.bounding_box.ulx for cell in cell_item if isinstance(cell.bounding_box, BoundingBox)])
+            ulx = min(cell.bounding_box.ulx for cell in cell_item if isinstance(cell.bounding_box, BoundingBox))
 
-            uly = min([cell.bounding_box.uly for cell in cell_item if isinstance(cell.bounding_box, BoundingBox)])
+            uly = min(cell.bounding_box.uly for cell in cell_item if isinstance(cell.bounding_box, BoundingBox))
 
-            lrx = max([cell.bounding_box.lrx for cell in cell_item if isinstance(cell.bounding_box, BoundingBox)])
+            lrx = max(cell.bounding_box.lrx for cell in cell_item if isinstance(cell.bounding_box, BoundingBox))
 
-            lry = max([cell.bounding_box.lry for cell in cell_item if isinstance(cell.bounding_box, BoundingBox)])
+            lry = max(cell.bounding_box.lry for cell in cell_item if isinstance(cell.bounding_box, BoundingBox))
 
             if pubtables_like:
-                tables = image.get_annotation(category_names=LayoutType.table)
+                tables = image.get_annotation(category_names=LayoutType.TABLE)
                 if not tables:
                     raise ValueError("pubtables_like = True requires table")
                 table = tables[0]
 
-                if item_type == LayoutType.row:
+                if item_type == LayoutType.ROW:
                     if table.bounding_box:
                         ulx = table.bounding_box.ulx + 1.0
                         lrx = table.bounding_box.lrx - 1.0
@@ -204,31 +199,28 @@ def _add_items(image: Image, item_type: str, categories_name_as_key: Dict[str, s
                         lry = table.bounding_box.lry - 1.0
 
             item_ann = ImageAnnotation(
-                category_id=categories_name_as_key[TableType.item],
-                category_name=TableType.item,
+                category_id=categories_name_as_key[TableType.ITEM],
+                category_name=TableType.ITEM,
                 bounding_box=BoundingBox(absolute_coords=True, ulx=ulx, uly=uly, lrx=lrx, lry=lry),
             )
             item_sub_ann = CategoryAnnotation(category_name=item_type)
-            item_ann.dump_sub_category(TableType.item, item_sub_ann, image.image_id)
+            item_ann.dump_sub_category(TableType.ITEM, item_sub_ann, image.image_id)
             image.dump(item_ann)
 
     if pubtables_like:  # pubtables_like:
-        items = image.get_annotation(category_names=TableType.item)
-        item_type_anns = [ann for ann in items if ann.get_sub_category(TableType.item).category_name == item_type]
+        items = image.get_annotation(category_names=TableType.ITEM)
+        item_type_anns = [ann for ann in items if ann.get_sub_category(TableType.ITEM).category_name == item_type]
         item_type_anns.sort(
-            key=lambda x: x.bounding_box.cx  # type: ignore
-            if item_type == LayoutType.column
-            else x.bounding_box.cy  # type: ignore
+            key=lambda x: (x.bounding_box.cx if item_type == LayoutType.COLUMN else x.bounding_box.cy)  # type: ignore
         )
         if table.bounding_box:
-            tmp_item_xy = table.bounding_box.uly + 1.0 if item_type == LayoutType.row else table.bounding_box.ulx + 1.0
+            tmp_item_xy = table.bounding_box.uly + 1.0 if item_type == LayoutType.ROW else table.bounding_box.ulx + 1.0
         for idx, item in enumerate(item_type_anns):
             with MappingContextManager(
                 dp_name=image.file_name,
                 filter_level="bounding box",
                 image_annotation={"category_name": item.category_name, "annotation_id": item.annotation_id},
             ):
-
                 box = item.bounding_box
                 if box:
                     tmp_next_item_xy = 0.0
@@ -237,22 +229,22 @@ def _add_items(image: Image, item_type: str, categories_name_as_key: Dict[str, s
                         if next_box:
                             tmp_next_item_xy = (
                                 (box.lry + next_box.uly) / 2
-                                if item_type == LayoutType.row
+                                if item_type == LayoutType.ROW
                                 else (box.lrx + next_box.ulx) / 2
                             )
                     else:
                         if table.bounding_box:
                             tmp_next_item_xy = (
                                 table.bounding_box.lry - 1.0
-                                if item_type == LayoutType.row
+                                if item_type == LayoutType.ROW
                                 else table.bounding_box.lrx - 1.0
                             )
 
                     new_embedding_box = BoundingBox(
-                        ulx=box.ulx if item_type == LayoutType.row else tmp_item_xy,
-                        uly=tmp_item_xy if item_type == LayoutType.row else box.uly,
-                        lrx=box.lrx if item_type == LayoutType.row else tmp_next_item_xy,
-                        lry=tmp_next_item_xy if item_type == LayoutType.row else box.lry,
+                        ulx=box.ulx if item_type == LayoutType.ROW else tmp_item_xy,
+                        uly=tmp_item_xy if item_type == LayoutType.ROW else box.uly,
+                        lrx=box.lrx if item_type == LayoutType.ROW else tmp_next_item_xy,
+                        lry=tmp_next_item_xy if item_type == LayoutType.ROW else box.lry,
                         absolute_coords=True,
                     )
                     item.bounding_box = new_embedding_box
@@ -262,12 +254,15 @@ def _add_items(image: Image, item_type: str, categories_name_as_key: Dict[str, s
     return image
 
 
-def row_col_cell_ids(tiling: List[List[int]]) -> List[Tuple[int, int, int]]:
+def row_col_cell_ids(tiling: list[list[int]]) -> list[tuple[int, int, int]]:
     """
     Infers absolute rows and columns for every cell from the tiling of a table.
 
-    :param tiling: A list of list of tiling of a table as returned from the `_tile_table`
-    :return: A list of 3-tuples with row number, column number and cell id.
+    Args:
+        tiling: A list of lists of tiling of a table as returned from the `_tile_table`.
+
+    Returns:
+        A list of `3-tuples` with row number, column number, and cell id.
     """
     indices = sorted(
         [(i + 1, j + 1, cell_id) for i, row in enumerate(tiling) for j, cell_id in enumerate(row)], key=lambda x: x[2]
@@ -278,23 +273,24 @@ def row_col_cell_ids(tiling: List[List[int]]) -> List[Tuple[int, int, int]]:
     return rows_col_cell_ids
 
 
-def embedding_in_image(dp: Image, html: List[str], categories_name_as_key: Dict[str, str]) -> Image:
+def embedding_in_image(dp: Image, html: list[str], categories_name_as_key: dict[ObjectTypes, int]) -> Image:
     """
-    Generating an image, that resembles the output of an analyzer. The layout of the image is table spanning
-    the full page, i.e. there is one table image annotation. Moreover, the table annotation has an image, with cells
-    as image annotations.
+    Generating an image that resembles the output of an analyzer. The layout of the image is a table spanning the full
+    page, i.e. there is one table image annotation. Moreover, the table annotation has an image, with cells as image
+    annotations.
 
-    :param dp: Image
-    :param html: list with html tags
-    :param categories_name_as_key: category dictionary with all possible annotations
-    :return: Image
+    Args:
+        dp: Image html: List with html tags. categories_name_as_key: Category dictionary with all possible annotations.
+
+    Returns:
+        Image
     """
     image = Image(file_name=dp.file_name, location=dp.location, external_id=dp.image_id + "image")
     image.image = dp.image
     image.set_width_height(dp.width, dp.height)
     table_ann = ImageAnnotation(
-        category_name=LayoutType.table,
-        category_id=categories_name_as_key[LayoutType.table],
+        category_name=LayoutType.TABLE,
+        category_id=categories_name_as_key[LayoutType.TABLE],
         bounding_box=BoundingBox(absolute_coords=True, ulx=0.0, uly=0.0, lrx=dp.width, lry=dp.height),
     )
     image.dump(table_ann)
@@ -304,40 +300,41 @@ def embedding_in_image(dp: Image, html: List[str], categories_name_as_key: Dict[
     # node.
     html.insert(0, "<table>")
     html.append("</table>")
-    if CellType.header not in categories_name_as_key:
+    if CellType.HEADER not in categories_name_as_key:
         html.remove("<thead>")
         html.remove("</thead>")
         if "<tbody>" in html and "</tbody>" in html:
             html.remove("<tbody>")
             html.remove("</tbody>")
 
-    html_ann = ContainerAnnotation(category_name=TableType.html, value=html)
-    table_ann.dump_sub_category(TableType.html, html_ann)
+    html_ann = ContainerAnnotation(category_name=TableType.HTML, value=html)
+    table_ann.dump_sub_category(TableType.HTML, html_ann)
     for ann in dp.get_annotation():
         image.dump(ann)
         assert table_ann.image
         table_ann.image.dump(ann)
-        table_ann.dump_relationship(Relationships.child, ann.annotation_id)
+        table_ann.dump_relationship(Relationships.CHILD, ann.annotation_id)
 
     return image
 
 
 def nth_index(iterable: Iterable[str], value: str, n: int) -> Optional[int]:
     """
-    Returns the position of the n-th string value in an iterable, e.g. a list
+    Returns the position of the n-th string value in an iterable, e.g. a list.
 
-    :param iterable: e.g. list
-    :param value: any value
-    :param n: n-th value
-    :return: position if n-th value exists else None
+    Args:
+        iterable: e.g. list value: Any value `n`: `n-th` value
+
+    Returns:
+        Position if `n-th` value exists else `None`.
     """
     matches = (idx for idx, val in enumerate(iterable) if val == value)
     return next(itertools.islice(matches, n - 1, n), None)
 
 
 def pub_to_image_uncur(  # pylint: disable=R0914
-    dp: JsonDict,
-    categories_name_as_key: Dict[str, str],
+    dp: PubtabnetDict,
+    categories_name_as_key: dict[ObjectTypes, int],
     load_image: bool,
     fake_score: bool,
     rows_and_cols: bool,
@@ -349,19 +346,21 @@ def pub_to_image_uncur(  # pylint: disable=R0914
     Map a datapoint of annotation structure as given in the Pubtabnet dataset to an Image structure.
     <https://github.com/ibm-aur-nlp/PubTabNet>
 
-    :param dp: A datapoint in serialized coco format.
-    :param categories_name_as_key: A dict of categories, e.g. DatasetCategories.get_categories(name_as_key=True)
-    :param load_image: If `True` it will load image to `Image.image`
-    :param fake_score: If dp does not contain a score, a fake score with uniform random variables in (0,1)
+    Args:
+        dp: A datapoint in serialized Pubtabnet format.
+        categories_name_as_key: A dict of categories, e.g. `DatasetCategories.get_categories(name_as_key=True)`
+        load_image: If `True` it will load image to `Image.image`
+        fake_score: If dp does not contain a score, a fake score with uniform random variables in (0,1)
                        will be added.
-    :param rows_and_cols: If set to `True`, synthetic "ITEM" ImageAnnotations will be added.  Each item has a
-                          sub-category "row_col" that is equal to "ROW" or "COL".
-    :param dd_pipe_like: This will generate an image identical to the output of the dd analyzer (e.g. table and words
+        rows_and_cols: If set to `True`, synthetic `ITEM` ImageAnnotations will be added.  Each item has a
+                          sub-category "row_col" that is equal to `ROW` or `COL`.
+        dd_pipe_like: This will generate an image identical to the output of the dd analyzer (e.g. table and words
                          annotations as well as sub annotations and relationships will be generated)
-    :param is_fintabnet: Set `True`, if this mapping is used for generating fintabnet datapoints.
-    :param pubtables_like: Set `True`, the area covering the table will be tiled with non overlapping rows and columns
+        is_fintabnet: Set `True`, if this mapping is used for generating Fintabnet datapoints.
+        pubtables_like: Set `True`, the area covering the table will be tiled with non overlapping rows and columns
                            without leaving empty space
-    :return: Image
+    Returns:
+        Image
     """
 
     if dd_pipe_like:
@@ -394,10 +393,9 @@ def pub_to_image_uncur(  # pylint: disable=R0914
         return None
 
     with MappingContextManager(str(idx)) as mapping_context:
-
         max_rs, max_cs = 0, 0
         if idx is None:
-            raise ValueError("No valid datapoint external id")
+            raise TypeError("imgid is None but must be a string")
 
         image = Image(file_name=os.path.split(dp["filename"])[1], location=dp["filename"], external_id=idx)
 
@@ -405,7 +403,7 @@ def pub_to_image_uncur(  # pylint: disable=R0914
             np_image = load_image_from_file(dp["filename"])
         if is_file_extension(dp["filename"], ".pdf"):
             pdf_bytes = load_bytes_from_pdf_file(dp["filename"])
-            np_image = convert_pdf_bytes_to_np_array_v2(pdf_bytes)
+            np_image = convert_pdf_bytes_to_np_array_v2(pdf_bytes, dpi=200)
             dp = _convert_boxes(dp, np_image.shape[0])
 
         if load_image and np_image is not None:
@@ -415,60 +413,56 @@ def pub_to_image_uncur(  # pylint: disable=R0914
 
         table_ann: Optional[ImageAnnotation] = None
         if is_fintabnet:  # cannot use for synthetic table ann creation
-            table_ann = _get_table_annotation(dp, categories_name_as_key[LayoutType.table])
+            table_ann = _get_table_annotation(dp, categories_name_as_key[LayoutType.TABLE])
             image.dump(table_ann)
 
-        for idx, value in enumerate(
+        for idx, (row_col_cell_id, cell, row_span, col_span) in enumerate(
             zip(rows_cols_cell_ids[::-1], dp["html"]["cells"][::-1], row_spans[::-1], col_spans[::-1])
         ):
-            row_col_cell_id = value[0]
             row_number, col_number, cell_id = row_col_cell_id[0], row_col_cell_id[1], row_col_cell_id[2]
-            cell = value[1]
-            row_span = value[2]
-            col_span = value[3]
 
             if "bbox" in cell:  # empty cells have no box
                 ulx, uly, lrx, lry = list(map(float, cell["bbox"]))
                 cell_bounding_box = BoundingBox(absolute_coords=True, ulx=ulx, uly=uly, lrx=lrx, lry=lry)
                 cell_ann = ImageAnnotation(
-                    category_name=LayoutType.cell,
+                    category_name=LayoutType.CELL,
                     bounding_box=cell_bounding_box,
-                    category_id=categories_name_as_key[LayoutType.cell],
+                    category_id=categories_name_as_key[LayoutType.CELL],
                     score=maybe_get_fake_score(fake_score),
                 )
                 cell_ann.dump_sub_category(
-                    CellType.row_number,
-                    CategoryAnnotation(category_name=CellType.row_number, category_id=str(row_number)),
+                    CellType.ROW_NUMBER,
+                    CategoryAnnotation(category_name=CellType.ROW_NUMBER, category_id=row_number),
                     image.image_id,
                 )
                 cell_ann.dump_sub_category(
-                    CellType.column_number,
-                    CategoryAnnotation(category_name=CellType.column_number, category_id=str(col_number)),
+                    CellType.COLUMN_NUMBER,
+                    CategoryAnnotation(category_name=CellType.COLUMN_NUMBER, category_id=col_number),
                     image.image_id,
                 )
                 cell_ann.dump_sub_category(
-                    CellType.row_span,
-                    CategoryAnnotation(category_name=CellType.row_span, category_id=str(row_span)),
+                    CellType.ROW_SPAN,
+                    CategoryAnnotation(category_name=CellType.ROW_SPAN, category_id=row_span),  # type: ignore
                     image.image_id,
                 )
                 cell_ann.dump_sub_category(
-                    CellType.column_span,
-                    CategoryAnnotation(category_name=CellType.column_span, category_id=str(col_span)),
+                    CellType.COLUMN_SPAN,
+                    CategoryAnnotation(category_name=CellType.COLUMN_SPAN, category_id=col_span),  # type: ignore
                     image.image_id,
                 )
                 if (
-                    int(cell_ann.get_sub_category(CellType.row_span).category_id) > 1
-                    or int(cell_ann.get_sub_category(CellType.column_span).category_id) > 1
+                    cell_ann.get_sub_category(CellType.ROW_SPAN).category_id > 1
+                    or cell_ann.get_sub_category(CellType.COLUMN_SPAN).category_id > 1
                 ):
                     cell_ann.dump_sub_category(
-                        CellType.spanning,
-                        CategoryAnnotation(category_name=CellType.spanning),
+                        CellType.SPANNING,
+                        CategoryAnnotation(category_name=CellType.SPANNING),
                         image.image_id,
                     )
                 else:
                     cell_ann.dump_sub_category(
-                        CellType.spanning,
-                        CategoryAnnotation(category_name=LayoutType.cell),
+                        CellType.SPANNING,
+                        CategoryAnnotation(category_name=LayoutType.CELL),
                         image.image_id,
                     )
 
@@ -476,13 +470,13 @@ def pub_to_image_uncur(  # pylint: disable=R0914
                 max_cs = max(max_cs, col_span)  # type: ignore
 
                 if _has_header:
-                    category_name = CellType.header if cell_id <= end_of_header else CellType.body
+                    category_name = CellType.HEADER if cell_id <= end_of_header else CellType.BODY
                     cell_ann.dump_sub_category(
-                        CellType.header, CategoryAnnotation(category_name=category_name), image.image_id
+                        CellType.HEADER, CategoryAnnotation(category_name=category_name), image.image_id
                     )
                 image.dump(cell_ann)
                 if table_ann is not None:
-                    table_ann.dump_relationship(Relationships.child, cell_ann.annotation_id)
+                    table_ann.dump_relationship(Relationships.CHILD, cell_ann.annotation_id)
 
                 if dd_pipe_like:
                     tokens = cell["tokens"]
@@ -492,47 +486,47 @@ def pub_to_image_uncur(  # pylint: disable=R0914
                     text = "".join(tokens)
                     # we are not separating each word but view the full table content as one word
                     word = ImageAnnotation(
-                        category_name=LayoutType.word,
-                        category_id=categories_name_as_key[LayoutType.word],
+                        category_name=LayoutType.WORD,
+                        category_id=categories_name_as_key[LayoutType.WORD],
                         bounding_box=cell_bounding_box,
                     )
-                    text_container = ContainerAnnotation(category_name=WordType.characters, value=text)
-                    word.dump_sub_category(WordType.characters, text_container)
-                    reading_order = CategoryAnnotation(category_name=Relationships.reading_order, category_id="1")
-                    word.dump_sub_category(Relationships.reading_order, reading_order)
+                    text_container = ContainerAnnotation(category_name=WordType.CHARACTERS, value=text)
+                    word.dump_sub_category(WordType.CHARACTERS, text_container)
+                    reading_order = CategoryAnnotation(category_name=Relationships.READING_ORDER, category_id=1)
+                    word.dump_sub_category(Relationships.READING_ORDER, reading_order)
                     image.dump(word)
-                    cell_ann.dump_relationship(Relationships.child, word.annotation_id)
+                    cell_ann.dump_relationship(Relationships.CHILD, word.annotation_id)
 
                     index = nth_index(html, "<td>", number_of_cells - idx)
                     if index:
                         html.insert(index + 1, cell_ann.annotation_id)
 
-        summary_ann = SummaryAnnotation(external_id=image.image_id + "SUMMARY")
+        summary_ann = CategoryAnnotation(category_name=SummaryType.SUMMARY)
         summary_ann.dump_sub_category(
-            TableType.number_of_rows,
-            CategoryAnnotation(category_name=TableType.number_of_rows, category_id=str(number_of_rows)),
+            TableType.NUMBER_OF_ROWS,
+            CategoryAnnotation(category_name=TableType.NUMBER_OF_ROWS, category_id=number_of_rows),
             image.image_id,
         )
         summary_ann.dump_sub_category(
-            TableType.number_of_columns,
-            CategoryAnnotation(category_name=TableType.number_of_columns, category_id=str(number_of_cols)),
+            TableType.NUMBER_OF_COLUMNS,
+            CategoryAnnotation(category_name=TableType.NUMBER_OF_COLUMNS, category_id=number_of_cols),
             image.image_id,
         )
         summary_ann.dump_sub_category(
-            TableType.max_row_span,
-            CategoryAnnotation(category_name=TableType.max_row_span, category_id=str(max_rs)),
+            TableType.MAX_ROW_SPAN,
+            CategoryAnnotation(category_name=TableType.MAX_ROW_SPAN, category_id=max_rs),
             image.image_id,
         )
         summary_ann.dump_sub_category(
-            TableType.max_col_span,
-            CategoryAnnotation(category_name=TableType.max_col_span, category_id=str(max_cs)),
+            TableType.MAX_COL_SPAN,
+            CategoryAnnotation(category_name=TableType.MAX_COL_SPAN, category_id=max_cs),
             image.image_id,
         )
         image.summary = summary_ann
 
         if rows_and_cols or dd_pipe_like:
-            image = _add_items(image, LayoutType.row, categories_name_as_key, pubtables_like)
-            image = _add_items(image, LayoutType.column, categories_name_as_key, pubtables_like)
+            image = _add_items(image, LayoutType.ROW, categories_name_as_key, pubtables_like)
+            image = _add_items(image, LayoutType.COLUMN, categories_name_as_key, pubtables_like)
 
         if dd_pipe_like:
             image = embedding_in_image(image, html, categories_name_as_key)
