@@ -21,7 +21,7 @@ Module for language detection pipeline component
 from typing import Optional, Sequence
 
 from ..datapoint.image import Image, MetaAnnotation
-from ..datapoint.view import ImageDefaults, Page
+from ..datapoint.view import IMAGE_DEFAULTS, Page
 from ..extern.base import LanguageDetector, ObjectDetector
 from ..utils.error import ImageError
 from ..utils.settings import PageType, TypeOrStr, get_type
@@ -75,11 +75,11 @@ class LanguageDetectionService(PipelineComponent):
 
         self.predictor = language_detector
         self.text_detector = text_detector
-        self.text_container = get_type(text_container) if text_container is not None else ImageDefaults.TEXT_CONTAINER
+        self.text_container = get_type(text_container) if text_container is not None else IMAGE_DEFAULTS.TEXT_CONTAINER
         self.floating_text_block_categories = (
             tuple(get_type(text_block) for text_block in floating_text_block_categories)
             if (floating_text_block_categories is not None)
-            else ()
+            else IMAGE_DEFAULTS.FLOATING_TEXT_BLOCK_CATEGORIES
         )
 
         super().__init__(self._get_name(self.predictor.name))
@@ -109,7 +109,7 @@ class LanguageDetectionService(PipelineComponent):
             text = " ".join((result.text for result in detect_result_list if result.text is not None))
         predict_result = self.predictor.predict(text)
         self.dp_manager.set_summary_annotation(
-            PageType.LANGUAGE, PageType.LANGUAGE, 1, predict_result.text, predict_result.score
+            PageType.LANGUAGE, PageType.LANGUAGE, 1, predict_result.class_name, predict_result.score
         )
 
     def clone(self) -> PipelineComponent:
