@@ -228,8 +228,8 @@ class OrderGenerator:
         columns: list[BoundingBox] = []
         anns.sort(
             key=lambda x: (
-                x.bounding_box.transform(image_width, image_height).cy,  # type: ignore
-                x.bounding_box.transform(image_width, image_height).cx,  # type: ignore
+                x.get_bounding_box(image_id).transform(image_width, image_height).cy,
+                x.get_bounding_box(image_id).transform(image_width, image_height).cx,
             )
         )
         for ann in anns:
@@ -309,7 +309,9 @@ class OrderGenerator:
         filtered_blocks: Sequence[tuple[int, str]]
         for idx in range(max_block_number + 1):
             filtered_blocks = list(filter(lambda x: x[0] == idx, blocks))  # type: ignore # pylint: disable=W0640
-            sorted_blocks.extend(self._sort_anns_grouped_by_blocks(filtered_blocks, anns, image_width, image_height))
+            sorted_blocks.extend(
+                self._sort_anns_grouped_by_blocks(filtered_blocks, anns, image_width, image_height, image_id)
+            )
         reading_blocks = [(idx + 1, block[1]) for idx, block in enumerate(sorted_blocks)]
 
         if logger.isEnabledFor(DEBUG):
@@ -346,7 +348,11 @@ class OrderGenerator:
 
     @staticmethod
     def _sort_anns_grouped_by_blocks(
-        block: Sequence[tuple[int, str]], anns: Sequence[ImageAnnotation], image_width: float, image_height: float
+        block: Sequence[tuple[int, str]],
+        anns: Sequence[ImageAnnotation],
+        image_width: float,
+        image_height: float,
+        image_id: Optional[str] = None,
     ) -> list[tuple[int, str]]:
         if not block:
             return []
@@ -356,8 +362,8 @@ class OrderGenerator:
         block_anns = [ann for ann in anns if ann.annotation_id in ann_ids]
         block_anns.sort(
             key=lambda x: (
-                round(x.bounding_box.transform(image_width, image_height).uly, 2),  # type: ignore
-                round(x.bounding_box.transform(image_width, image_height).ulx, 2),  # type: ignore
+                round(x.get_bounding_box(image_id).transform(image_width, image_height).uly, 2),
+                round(x.get_bounding_box(image_id).transform(image_width, image_height).ulx, 2),
             )
         )
         return [(block_number, ann.annotation_id) for ann in block_anns]
