@@ -16,7 +16,7 @@
 # limitations under the License.
 
 """
-Some useful contextmanagers for various tasks
+Contextmanagers for various tasks
 """
 
 import subprocess
@@ -26,27 +26,38 @@ from glob import iglob
 from os import path, remove
 from tempfile import NamedTemporaryFile
 from time import perf_counter as timer
-from typing import Any, Generator, Iterator, Optional, Tuple, Union
+from typing import Any, Generator, Iterator, Optional, Union
 
 import numpy as np
-from cv2 import imwrite
 
-from .detection_types import ImageType
-from .logger import logger
+from .logger import LoggingRecord, logger
+from .types import B64, B64Str, PixelValues
+from .viz import viz_handler
 
 __all__ = ["timeout_manager", "save_tmp_file", "timed_operation"]
 
 
 @contextmanager
-def timeout_manager(proc, seconds: Optional[int] = None) -> Iterator[str]:  # type: ignore
+def timeout_manager(proc: Any, seconds: Optional[int] = None) -> Iterator[str]:
     """
-    Manager for time handling while some process being called
+    Manager for time handling while some process is being called.
 
-       with timeout_manager(some_process,60) as timeout:
-           ...
+    Example:
+        ```python
+        with timeout_manager(some_process, 60) as timeout:
+            ...
+        ```
+
+    Args:
+        proc: Process.
+        seconds: Seconds to wait.
+
+    Yields:
+        str: Error string from the process.
+
+    Raises:
+        RuntimeError: If the process times out.
 
-    :param proc: process
-    :param seconds: seconds to wait
 
     """
     try:
@@ -61,7 +72,7 @@ def timeout_manager(proc, seconds: Optional[int] = None) -> Iterator[str]:  # ty
             proc.terminate()
             proc.kill()
             proc.returncode = -1
-            raise RuntimeError("Tesseract process timeout")  # pylint: disable=W0707
+            raise RuntimeError(f"timeout for process id: {proc.pid}")  # pylint: disable=W0707
     finally:
         if proc.stdin is not None:
             proc.stdin.close()
@@ -72,15 +83,22 @@ def timeout_manager(proc, seconds: Optional[int] = None) -> Iterator[str]:  # ty
 
 
 @contextmanager
-def save_tmp_file(image: Union[str, ImageType, bytes], prefix: str) -> Iterator[Tuple[str, str]]:
+def save_tmp_file(image: Union[B64Str, PixelValues, B64], prefix: str) -> Iterator[tuple[str, str]]:
     """
-    Save image temporarily and handle the clean-up once not necessary anymore
+    Save image temporarily and handle the clean-up once not necessary anymore.
 
-        with save_tmp_file(some_np_image,"tmp") as (tmp_name, input_file_name):
-            ....
+    Args:
+        image: Image as string or `np.array`.
+        prefix: Prefix of the temp file name.
 
-    :param image: image as string or numpy array
-    :param prefix: prefix of the temp file name
+    Yields:
+        Tuple containing the temporary file name and the input file name.
+
+    Example:
+        ```python
+         with save_tmp_file(some_np_image, "tmp") as (tmp_name, input_file_name):
+             ...
+        ```
     """
     try:
         with NamedTemporaryFile(prefix=prefix, delete=False) as file:
@@ -88,8 +106,8 @@ def save_tmp_file(image: Union[str, ImageType, bytes], prefix: str) -> Iterator[
                 yield file.name, path.realpath(path.normpath(path.normcase(image)))
                 return
             if isinstance(image, (np.ndarray, np.generic)):
-                input_file_name = file.name + ".PNG"
-                imwrite(input_file_name, image)
+                input_file_name = file.name + "_input.PNG"
+                viz_handler.write_image(input_file_name, image)
                 yield file.name, input_file_name
             if isinstance(image, bytes):
                 input_file_name = file.name
@@ -112,15 +130,25 @@ def save_tmp_file(image: Union[str, ImageType, bytes], prefix: str) -> Iterator[
 @contextmanager
 def timed_operation(message: str, log_start: bool = False) -> Generator[Any, None, None]:
     """
-    Contextmanager with a timer. Can therefore be used in a with statement.
+    Context manager with a timer.
 
-    :param message: a log to print
-    :param log_start: whether to print also the beginning
+    Example:
+        ```python
+        with timed_operation(message="Your stdout message", log_start=True):
+            with open("log.txt", "a") as file:
+                ...
+        ```
+
+    Args:
+        message: A log to stdout.
+        log_start: Whether to print also the beginning.
+
+    Yields:
+        None
     """
 
-    assert len(message)
     if log_start:
-        logger.info("start task: %s ...", message)
+        logger.info(LoggingRecord(f"start task: {message} ..."))
     start = timer()
     yield
-    logger.info("%s total: %s sec.", message, round(timer() - start, 4))
+    logger.info(LoggingRecord(f"{message} total: {round(timer() - start, 4)} sec."))
