@@ -26,26 +26,30 @@ import functools
 import inspect
 from collections import defaultdict
 from datetime import datetime
-from typing import Callable, List, Optional
+from typing import Callable, Optional
 
-from .detection_types import T
-from .logger import logger
+from .logger import LoggingRecord, logger
+from .types import T
 
-__all__: List[str] = ["deprecated"]
+__all__: list[str] = ["deprecated"]
 
 # Copy and paste from https://github.com/tensorpack/tensorpack/blob/master/tensorpack/utils/develop.py
 
 _DEPRECATED_LOG_NUM = defaultdict(int)  # type: ignore
 
 
-def log_deprecated(name: str = "", text: str = "", eos: str = "", max_num_warnings: Optional[int] = None) -> None:
+def log_deprecated(name: str, text: str, eos: str = "", max_num_warnings: Optional[int] = None) -> None:
     """
-    Log deprecation warning.
+    Logs a deprecation warning.
 
-    :param name: name of the deprecated item.
-    :param text: information about the deprecation.
-    :param eos: end of service date such as "YYYY-MM-DD".
-    :param max_num_warnings: the maximum number of times to print this warning
+    Args:
+        name: Name of the deprecated item.
+        text: Information about the deprecation.
+        eos: End of service date such as "YYYY-MM-DD".
+        max_num_warnings: The maximum number of times to print this warning.
+
+    Note:
+        Either `name` or `text` must be provided.
     """
     assert name or text
     if eos:
@@ -64,25 +68,29 @@ def log_deprecated(name: str = "", text: str = "", eos: str = "", max_num_warnin
         if _DEPRECATED_LOG_NUM[info_msg] >= max_num_warnings:
             return
         _DEPRECATED_LOG_NUM[info_msg] += 1
-    logger.info("[Deprecated] %s", info_msg)
+    logger.info(LoggingRecord(f"[Deprecated] {info_msg}"))
 
 
 def deprecated(
     text: str = "", eos: str = "", max_num_warnings: Optional[int] = None
 ) -> Callable[[Callable[..., T]], Callable[..., T]]:
     """
+    Decorator to deprecate a function.
 
-    :param text: same as `log_deprecated`.
-    :param eos: same as `log_deprecated`.
-    :param max_num_warnings: same as `log_deprecated`.
+    Example:
+        ```python
+        @deprecated("Explanation of what to do instead.", "2017-11-4")
+        def foo(...):
+            pass
+        ```
 
-    :return: A decorator which deprecates the function.
+    Args:
+        text: Same as `log_deprecated`.
+        eos: Same as `log_deprecated`.
+        max_num_warnings: Same as `log_deprecated`.
 
-    **Example:**
-
-            @deprecated("Explanation of what to do instead.", "2017-11-4")
-            def foo(...):
-                pass
+    Returns:
+        A decorator which deprecates the function.
     """
 
     def get_location() -> str:
